@@ -134,8 +134,9 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
 #define ConnectedComponentsImageTag  "ConnectedComponents/Image"
 
   CacheView
+    *component_view,
     *image_view,
-    *component_view;
+    *object_view;
 
   CCObjectInfo
     *object;
@@ -170,6 +171,10 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
 
   ssize_t
     background_id,
+    connect4[2][2] = { { -1,  0 }, {  0, -1 } },
+    connect8[4][2] = { { -1, -1 }, { -1,  0 }, { -1,  1 }, {  0, -1 } },
+    dx,
+    dy,
     first,
     last,
     n,
@@ -236,16 +241,10 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
   image_view=AcquireVirtualCacheView(image,exception);
   for (n=0; n < (ssize_t) (connectivity > 4 ? 4 : 2); n++)
   {
-    ssize_t
-      connect4[2][2] = { { -1,  0 }, {  0, -1 } },
-      connect8[4][2] = { { -1, -1 }, { -1,  0 }, { -1,  1 }, {  0, -1 } },
-      dx,
-      dy;
-
     if (status == MagickFalse)
       continue;
-    dy=connectivity > 4 ? connect8[n][0] : connect4[n][0];
     dx=connectivity > 4 ? connect8[n][1] : connect4[n][1];
+    dy=connectivity > 4 ? connect8[n][0] : connect4[n][0];
     for (y=0; y < (ssize_t) image->rows; y++)
     {
       register const Quantum
@@ -343,12 +342,10 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
       }
     }
   }
-  image_view=DestroyCacheView(image_view);
   /*
     Label connected components.
   */
   n=0;
-  image_view=AcquireVirtualCacheView(image,exception);
   component_view=AcquireAuthenticCacheView(component_image,exception);
   for (y=0; y < (ssize_t) component_image->rows; y++)
   {
@@ -435,9 +432,9 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
       component_image=DestroyImage(component_image);
       ThrowImageException(ResourceLimitError,"TooManyObjects");
     }
+  background_id=0;
   min_threshold=0.0;
   max_threshold=0.0;
-  background_id=0;
   component_image->colors=(size_t) n;
   for (i=0; i < (ssize_t) component_image->colors; i++)
   {
@@ -464,99 +461,13 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
   if (artifact != (const char *) NULL)
     {
       /*
-        Merge object below area threshold.
+        Merge any object not within the min and max area threshold.
       */
       (void) sscanf(artifact,"%lf%*[ -]%lf",&min_threshold,&max_threshold);
-      component_view=AcquireAuthenticCacheView(component_image,exception);
       for (i=0; i < (ssize_t) component_image->colors; i++)
-      {
-        double
-          census;
-
-        RectangleInfo
-          bounding_box;
-
-        register ssize_t
-          j;
-
-        size_t
-          id;
-
-        if (status == MagickFalse)
-          continue;
-        if (((object[i].area >= min_threshold) &&
-             (object[i].area < max_threshold)) || (i == background_id))
-          continue;  /* keep object */
-        /*
-          Merge this object.
-        */
-        for (j=0; j < (ssize_t) component_image->colors; j++)
-          object[j].census=0;
-        bounding_box=object[i].bounding_box;
-        for (y=0; y < (ssize_t) bounding_box.height+2; y++)
-        {
-          register const Quantum
-            *magick_restrict p;
-
-          register ssize_t
-            x;
-
-          if (status == MagickFalse)
-            continue;
-          p=GetCacheViewVirtualPixels(component_view,bounding_box.x-1,
-            bounding_box.y+y-1,bounding_box.width+2,1,exception);
-          if (p == (const Quantum *) NULL)
-            {
-              status=MagickFalse;
-              continue;
-            }
-          for (x=0; x < (ssize_t) bounding_box.width+2; x++)
-          {
-            j=(ssize_t) GetPixelIndex(component_image,p);
-            if (j != i)
-              object[j].census++;
-            p+=GetPixelChannels(component_image);
-          }
-        }
-        census=0;
-        id=0;
-        for (j=0; j < (ssize_t) component_image->colors; j++)
-          if (census < object[j].census)
-            {
-              census=object[j].census;
-              id=(size_t) j;
-            }
-        object[id].area+=object[i].area;
-        object[i].area=0.0;
-        for (y=0; y < (ssize_t) bounding_box.height; y++)
-        {
-          register Quantum
-            *magick_restrict q;
-
-          register ssize_t
-            x;
-
-          if (status == MagickFalse)
-            continue;
-          q=GetCacheViewAuthenticPixels(component_view,bounding_box.x,
-            bounding_box.y+y,bounding_box.width,1,exception);
-          if (q == (Quantum *) NULL)
-            {
-              status=MagickFalse;
-              continue;
-            }
-          for (x=0; x < (ssize_t) bounding_box.width; x++)
-          {
-            if ((ssize_t) GetPixelIndex(component_image,q) == i)
-              SetPixelIndex(component_image,(Quantum) id,q);
-            q+=GetPixelChannels(component_image);
-          }
-          if (SyncCacheViewAuthenticPixels(component_view,exception) == MagickFalse)
-            status=MagickFalse;
-        }
-      }
-      component_view=DestroyCacheView(component_view);
-      (void) SyncImage(component_image,exception);
+        if (((object[i].area < min_threshold) ||
+             (object[i].area >= max_threshold)) && (i != background_id))
+          object[i].merge=MagickTrue;
     }
   artifact=GetImageArtifact(image,"connected-components:keep-colors");
   if (artifact != (const char *) NULL)
@@ -565,10 +476,10 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
         *p;
 
       /*
-        Keep selected objects based on color (make others transparent).
+        Keep selected objects based on color, merge others.
       */
       for (i=0; i < (ssize_t) component_image->colors; i++)
-        object[i].census=0;
+        object[i].merge=MagickTrue;
       for (p=artifact;  ; )
       {
         char
@@ -588,20 +499,11 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
         (void) QueryColorCompliance(color,AllCompliance,&pixel,exception);
         for (i=0; i < (ssize_t) component_image->colors; i++)
           if (IsFuzzyEquivalencePixelInfo(&object[i].color,&pixel) != MagickFalse)
-            object[i].census++;
+            object[i].merge=MagickFalse;
         if (*q == '\0')
           break;
         p=q+1;
       }
-      for (i=0; i < (ssize_t) component_image->colors; i++)
-      {
-        if (object[i].census == 0)
-          continue;
-        object[i].color.alpha_trait=BlendPixelTrait;
-        component_image->alpha_trait=BlendPixelTrait;
-        component_image->colormap[i].alpha_trait=BlendPixelTrait;
-        component_image->colormap[i].alpha=(MagickRealType) TransparentAlpha;
-      }
     }
   artifact=GetImageArtifact(image,"connected-components:keep-ids");
   if (artifact == (const char *) NULL)
@@ -609,10 +511,10 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
   if (artifact != (const char *) NULL)
     {
       /*
-        Keep selected objects based on ID (make others transparent).
+        Keep selected objects based on id, merge others.
       */
       for (i=0; i < (ssize_t) component_image->colors; i++)
-        object[i].census=0;
+        object[i].merge=MagickTrue;
       for (c=(char *) artifact; *c != '\0';)
       {
         while ((isspace((int) ((unsigned char) *c)) != 0) || (*c == ','))
@@ -631,38 +533,36 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
           }
         step=(ssize_t) (first > last ? -1 : 1);
         for ( ; first != (last+step); first+=step)
-          object[first].census++;
-      }
-      for (i=0; i < (ssize_t) component_image->colors; i++)
-      {
-        if (object[i].census != 0)
-          continue;
-        object[i].color.alpha_trait=BlendPixelTrait;
-        component_image->alpha_trait=BlendPixelTrait;
-        component_image->colormap[i].alpha_trait=BlendPixelTrait;
-        component_image->colormap[i].alpha=(MagickRealType) TransparentAlpha;
+          object[first].merge=MagickFalse;
       }
     }
   artifact=GetImageArtifact(image,"connected-components:keep-top");
   if (artifact != (const char *) NULL)
     {
+      CCObjectInfo
+        *top_objects;
+
       ssize_t
         top_ids;
 
       /*
-        Keep top objects with most area (make others transparent).
+        Keep top objects.
       */
       top_ids=(ssize_t) StringToDouble(artifact,(char **) NULL);
-      qsort((void *) object,component_image->colors,sizeof(*object),
+      top_objects=(CCObjectInfo *) AcquireQuantumMemory(component_image->colors,
+        sizeof(*top_objects));
+      if (top_objects == (CCObjectInfo *) NULL)
+        {
+          object=(CCObjectInfo *) RelinquishMagickMemory(object);
+          component_image=DestroyImage(component_image);
+          ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
+        }
+      (void) memcpy(top_objects,object,component_image->colors*sizeof(*object));
+      qsort((void *) top_objects,component_image->colors,sizeof(*top_objects),
         CCObjectInfoCompare);
-      for (i=top_ids; i < (ssize_t) component_image->colors; i++)
-      {
-        object[object[i].id].color.alpha_trait=BlendPixelTrait;
-        component_image->alpha_trait=BlendPixelTrait;
-        component_image->colormap[object[i].id].alpha_trait=BlendPixelTrait;
-        component_image->colormap[object[i].id].alpha=(MagickRealType)
-          TransparentAlpha;
-      }
+      for (i=top_ids+1; i < (ssize_t) component_image->colors; i++)
+        object[top_objects[i].id].merge=MagickTrue;
+      top_objects=(CCObjectInfo *) RelinquishMagickMemory(top_objects);
     }
   artifact=GetImageArtifact(image,"connected-components:remove-colors");
   if (artifact != (const char *) NULL)
@@ -671,10 +571,8 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
         *p;
 
       /*
-        Keep selected objects based on color (make others transparent).
+        Remove selected objects based on color, keep others.
       */
-      for (i=0; i < (ssize_t) component_image->colors; i++)
-        object[i].census=0;
       for (p=artifact;  ; )
       {
         char
@@ -694,55 +592,152 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
         (void) QueryColorCompliance(color,AllCompliance,&pixel,exception);
         for (i=0; i < (ssize_t) component_image->colors; i++)
           if (IsFuzzyEquivalencePixelInfo(&object[i].color,&pixel) != MagickFalse)
-            object[i].census++;
+            object[i].merge=MagickTrue;
         if (*q == '\0')
           break;
         p=q+1;
       }
-      for (i=0; i < (ssize_t) component_image->colors; i++)
-      {
-        if (object[i].census != 0)
-          continue;
-        object[i].color.alpha_trait=BlendPixelTrait;
-        component_image->alpha_trait=BlendPixelTrait;
-        component_image->colormap[i].alpha_trait=BlendPixelTrait;
-        component_image->colormap[i].alpha=(MagickRealType) TransparentAlpha;
-      }
     }
   artifact=GetImageArtifact(image,"connected-components:remove-ids");
   if (artifact == (const char *) NULL)
     artifact=GetImageArtifact(image,"connected-components:remove");
   if (artifact != (const char *) NULL)
+    for (c=(char *) artifact; *c != '\0';)
     {
       /*
-        Remove these object (make them transparent).
+        Remove selected objects based on id, keep others.
       */
-      for (c=(char *) artifact; *c != '\0';)
-      {
-        while ((isspace((int) ((unsigned char) *c)) != 0) || (*c == ','))
-          c++;
-        first=(ssize_t) strtol(c,&c,10);
-        if (first < 0)
-          first+=(ssize_t) component_image->colors;
-        last=first;
-        while (isspace((int) ((unsigned char) *c)) != 0)
-          c++;
-        if (*c == '-')
-          {
-            last=(ssize_t) strtol(c+1,&c,10);
-            if (last < 0)
-              last+=(ssize_t) component_image->colors;
-          }
-        step=(ssize_t) (first > last ? -1 : 1);
-        for ( ; first != (last+step); first+=step)
+      while ((isspace((int) ((unsigned char) *c)) != 0) || (*c == ','))
+        c++;
+      first=(ssize_t) strtol(c,&c,10);
+      if (first < 0)
+        first+=(ssize_t) component_image->colors;
+      last=first;
+      while (isspace((int) ((unsigned char) *c)) != 0)
+        c++;
+      if (*c == '-')
         {
-          component_image->alpha_trait=BlendPixelTrait;
-          component_image->colormap[first].alpha_trait=BlendPixelTrait;
-          component_image->colormap[first].alpha=(MagickRealType)
-            TransparentAlpha;
+          last=(ssize_t) strtol(c+1,&c,10);
+          if (last < 0)
+            last+=(ssize_t) component_image->colors;
         }
+      step=(ssize_t) (first > last ? -1 : 1);
+      for ( ; first != (last+step); first+=step)
+        object[first].merge=MagickTrue;
+    }
+  /*
+    Merge any object not within the min and max area threshold.
+  */
+  component_view=AcquireAuthenticCacheView(component_image,exception);
+  object_view=AcquireVirtualCacheView(component_image,exception);
+  for (i=0; i < (ssize_t) component_image->colors; i++)
+  {
+    RectangleInfo
+      bounding_box;
+
+    register ssize_t
+      j;
+
+    size_t
+      id;
+
+    if (status == MagickFalse)
+      continue;
+    if ((object[i].merge == MagickFalse) || (i == background_id))
+      continue;  /* keep object */
+    /*
+      Merge this object.
+    */
+    for (j=0; j < (ssize_t) component_image->colors; j++)
+      object[j].census=0;
+    bounding_box=object[i].bounding_box;
+    for (y=0; y < (ssize_t) bounding_box.height; y++)
+    {
+      register const Quantum
+        *magick_restrict p;
+
+      register ssize_t
+        x;
+
+      if (status == MagickFalse)
+        continue;
+      p=GetCacheViewVirtualPixels(component_view,bounding_box.x,
+        bounding_box.y+y,bounding_box.width,1,exception);
+      if (p == (const Quantum *) NULL)
+        {
+          status=MagickFalse;
+          continue;
+        }
+      for (x=0; x < (ssize_t) bounding_box.width; x++)
+      {
+        if (status == MagickFalse)
+          continue;
+        j=(ssize_t) GetPixelIndex(component_image,p);
+        if (j == i)
+          for (n=0; n < (ssize_t) (connectivity > 4 ? 4 : 2); n++)
+          {
+            register const Quantum
+              *p;
+
+            /*
+              Compute area of adjacent objects.
+            */
+            if (status == MagickFalse)
+              continue;
+            dx=connectivity > 4 ? connect8[n][1] : connect4[n][1];
+            dy=connectivity > 4 ? connect8[n][0] : connect4[n][0];
+            p=GetCacheViewVirtualPixels(object_view,bounding_box.x+x+dx,
+              bounding_box.y+y+dy,1,1,exception);
+            if (p == (const Quantum *) NULL)
+              {
+                status=MagickFalse;
+                break;
+              }
+            j=(ssize_t) GetPixelIndex(component_image,p);
+            if (j != i)
+              object[j].census++;
+          }
+        p+=GetPixelChannels(component_image);
       }
     }
+    /*
+      Merge with object of greatest adjacent area.
+    */
+    id=0;
+    for (j=1; j < (ssize_t) component_image->colors; j++)
+      if (object[j].census > object[id].census)
+        id=(size_t) j;
+    object[id].area+=object[i].area;
+    object[i].area=0.0;
+    for (y=0; y < (ssize_t) bounding_box.height; y++)
+    {
+      register Quantum
+        *magick_restrict q;
+
+      register ssize_t
+        x;
+
+      if (status == MagickFalse)
+        continue;
+      q=GetCacheViewAuthenticPixels(component_view,bounding_box.x,
+        bounding_box.y+y,bounding_box.width,1,exception);
+      if (q == (Quantum *) NULL)
+        {
+          status=MagickFalse;
+          continue;
+        }
+      for (x=0; x < (ssize_t) bounding_box.width; x++)
+      {
+        if ((ssize_t) GetPixelIndex(component_image,q) == i)
+          SetPixelIndex(component_image,(Quantum) id,q);
+        q+=GetPixelChannels(component_image);
+      }
+      if (SyncCacheViewAuthenticPixels(component_view,exception) == MagickFalse)
+        status=MagickFalse;
+    }
+  }
+  object_view=DestroyCacheView(object_view);
+  component_view=DestroyCacheView(component_view);
   artifact=GetImageArtifact(image,"connected-components:mean-color");
   if (IsStringTrue(artifact) != MagickFalse)
     {
@@ -758,7 +753,7 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
       (objects != (CCObjectInfo **) NULL))
     {
       /*
-        Report statistics on unique object.
+        Report statistics on each unique object.
       */
       for (i=0; i < (ssize_t) component_image->colors; i++)
       {
@@ -841,7 +836,7 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
                   object[i].centroid.x,object[i].centroid.y,(double)
                   object[i].area,mean_color);
               }
-      }
+        }
     }
   if (objects == (CCObjectInfo **) NULL)
     object=(CCObjectInfo *) RelinquishMagickMemory(object);
